import { __extends } from "tslib";
import * as Vue from 'vue';
import { BaseModel } from '@redux-model/core';
import { ComposeAction } from '../actions/ComposeAction';
var Model = /** @class */ (function (_super) {
    __extends(Model, _super);
    function Model() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Model.prototype.useData = function (selector) {
        var _this = this;
        return Vue.computed(function () {
            return selector ? selector(_this.data) : _this.data;
        });
    };
    /**
     * The action which compose aysnchorize program and hold loading.
     * ```
     * class TestModel extends Model {
     *   updateRoom = this.compose(async (id: number) => {
     *     const roomId = await getRoomId(id);
     *     const userId = await getUserId(roomId);
     *
     *     this.changeState((state) => {
     *       state.push([userId, roomId]);
     *     });
     *   });
     * }
     *
     * const testModel = new TestModel();
     *
     * -------------
     *
     * // Hold loading
     * const loading = testModel.updateRoom.useLoading();
     * // Dispatch action
     * const promise = testModel.updateRoom(10);
     * ```
     */
    Model.prototype.compose = function (fn) {
        var action = new ComposeAction(this, fn);
        return action;
    };
    return Model;
}(BaseModel));
export { Model };
