import { __assign, __extends } from "tslib";
import axios from 'axios';
import { BaseHttpService, METHOD, storeHelper } from '@redux-model/core';
import PromiseListenCatch from 'promise-listen-catch';
import { RequestAction } from '../actions/RequestAction';
var HttpService = /** @class */ (function (_super) {
    __extends(HttpService, _super);
    function HttpService(config) {
        var _this = _super.call(this, config) || this;
        _this.httpHandler = axios.create(__assign({ baseURL: config.baseUrl, timeout: 20000, withCredentials: false, responseType: 'json' }, config.requestOptions));
        return _this;
    }
    HttpService.prototype.clone = function (config) {
        // @ts-ignore
        // @ts-expect-error
        return new HttpService(__assign(__assign({}, this.config), config));
    };
    HttpService.prototype.action = function (fn) {
        return new RequestAction(fn, this);
    };
    HttpService.prototype.runAction = function (action) {
        var _this = this;
        var config = this.config;
        config.beforeSend && config.beforeSend(action);
        // For service.xxxAsync(), prepare, success and fail are all empty string.
        var _a = action.type, prepare = _a.prepare, success = _a.success, fail = _a.fail;
        var source = axios.CancelToken.source();
        var requestOptions = __assign(__assign({ url: action.uri, params: action.query, cancelToken: source.token, method: action.method }, action.requestOptions), { headers: __assign(__assign({}, config.headers(action)), action.requestOptions.headers) });
        if (action.method !== METHOD.get && action.body) {
            requestOptions.data = action.body;
        }
        var successInvoked = false;
        prepare && storeHelper.dispatch(__assign(__assign({}, action), { type: prepare, loading: true, effect: action.onPrepare, after: action.afterPrepare, afterDuration: action.afterPrepareDuration }));
        var throttleData = this.getThrottleData(action, {
            url: requestOptions.url,
            actionName: action.actionName,
            method: action.method,
            body: action.body,
            query: action.query,
            headers: requestOptions.headers,
            transfer: action.throttle.transfer,
        });
        if (throttleData) {
            return throttleData;
        }
        var promise = this.httpHandler.request(requestOptions)
            .then(function (httpResponse) {
            if (config.isSuccess && !config.isSuccess(httpResponse)) {
                return Promise.reject({
                    response: httpResponse,
                });
            }
            if (config.onRespondSuccess) {
                config.onRespondSuccess(httpResponse);
            }
            var okAction = __assign(__assign({}, action), { type: success, loading: false, response: httpResponse.data, effect: action.onSuccess, after: action.afterSuccess, afterDuration: action.afterSuccessDuration });
            successInvoked = true;
            success && storeHelper.dispatch(okAction);
            _this.setThrottle(okAction);
            _this.triggerShowSuccess(okAction, action.successText);
            return Promise.resolve(okAction);
        })
            .catch(function (error) {
            if (successInvoked) {
                return Promise.reject(error);
            }
            var isCancel = axios.isCancel(error);
            var errorMessage;
            var httpStatus;
            var businessCode;
            if (isCancel) {
                errorMessage = error.message || 'Aborted';
            }
            else if (error.response) {
                var meta = {
                    httpStatus: error.response.status,
                };
                config.onRespondError(error.response, meta);
                errorMessage = action.failText || meta.message || 'Fail to request api';
                httpStatus = meta.httpStatus;
                businessCode = meta.businessCode;
            }
            else {
                errorMessage = error.message;
                if (/^timeout\sof\s\d+m?s\sexceeded$/i.test(errorMessage)) {
                    errorMessage = config.timeoutMessage ? config.timeoutMessage(errorMessage) : errorMessage;
                }
                else if (/Network\sError/i.test(errorMessage)) {
                    errorMessage = config.networkErrorMessage ? config.networkErrorMessage(errorMessage) : errorMessage;
                }
            }
            var errorResponse = __assign(__assign({}, action), { response: error.response && error.response.data, type: fail, loading: false, message: errorMessage, httpStatus: httpStatus,
                businessCode: businessCode, effect: action.onFail, after: action.afterFail, afterDuration: action.afterFailDuration });
            fail && storeHelper.dispatch(errorResponse);
            if (!isCancel) {
                _this.triggerShowError(errorResponse, action.hideError);
            }
            if (listener.canReject()) {
                return Promise.reject(errorResponse);
            }
            return;
        });
        var listener = new PromiseListenCatch(promise);
        // @ts-ignore
        var fakePromise = listener;
        fakePromise.cancel = source.cancel;
        return fakePromise;
    };
    return HttpService;
}(BaseHttpService));
export { HttpService };
