import * as diff from 'deep-diff';
import cloneDeep from 'clone';
var PATCH_KIND;
(function (PATCH_KIND) {
    PATCH_KIND["array"] = "A";
    PATCH_KIND["new"] = "N";
    PATCH_KIND["trash"] = "D";
    PATCH_KIND["edit"] = "E";
})(PATCH_KIND || (PATCH_KIND = {}));
var getDeepState = function (observer, paths, popLastPath) {
    if (popLastPath) {
        paths = paths.slice(0, paths.length - 1);
    }
    return paths.reduce(function (carry, value) {
        return carry[value];
    }, observer);
};
var getLastPath = function (paths) {
    return paths.slice(paths.length - 1)[0];
};
export var applyPatch = function (originalState, currentState, ovserver) {
    var patches = diff.diff(originalState, currentState);
    if (patches === undefined) {
        return;
    }
    for (var i = 0, len = patches.length; i < len; ++i) {
        var patch = patches[i];
        var patchPath = patch.path;
        switch (patch.kind) {
            case PATCH_KIND.array:
                if (patch.item.kind !== PATCH_KIND.new) {
                    throw new TypeError("Unknown diff kind '" + patch.item.kind + "' in array");
                }
                getDeepState(ovserver, patchPath, false)[patch.index] = cloneDeep(patch.item.rhs, false);
                break;
            case PATCH_KIND.trash:
                delete getDeepState(ovserver, patchPath, true)[getLastPath(patchPath)];
                break;
            case PATCH_KIND.edit:
            case PATCH_KIND.new:
                getDeepState(ovserver, patchPath, true)[getLastPath(patchPath)] = cloneDeep(patch.rhs, false);
                break;
            default:
                throw new TypeError('Unknown diff kind: ' + patch.kind);
        }
    }
};
